from tkinter import *
from tkinter import font
from tkinter import messagebox
from tkinter import filedialog
from othello.partie import Partie
from random import choice
import threading
import time
import platform
import pygame


class Fenetre(Tk):
    """
    L'object GUI du jeu d'Othello.
    """
    def __init__(self):
        """
        Méthode d'initialisation d'une fenêtre.
        """
        super().__init__()

        # Crée la barre de tâches.
        self.__create_taskbar()

        # Setup l'audio.
        pygame.mixer.pre_init(44100, -16, 5, 2048)
        pygame.mixer.init()
        pygame.init()
        self.music = pygame.mixer.Sound("focus.wav")
        self.music.set_volume(0.4)
        self.music.play(-1)
        self.gamepiece_sound = pygame.mixer.Sound("gamepiece.wav")
        self.astuce_sound = pygame.mixer.Sound("astuce.wav")
        self.winner_sound = pygame.mixer.Sound("winner.wav")
        self.loser_sound = pygame.mixer.Sound("loser.wav")

        # Une référence à la partie en cours.
        self.partie = None

        # La dimension de la planche.
        self.nb_cases = None

        # Un décalage qui dépend de la dimension de la planche.
        self.decalage = None

        # Les types des joueurs noir et blanc.
        self.type_joueur_noir = "humain"
        self.type_joueur_blanc = "humain"

        # Un dictionnaire contenant un text field pour chaque case du jeu.
        self.pieces = {}

        # Des String Variables qui seront utilisés par des radio buttons.
        self.difficultee_noir = StringVar()
        self.difficultee_noir.set("facile")
        self.difficultee_blanc = StringVar()
        self.difficultee_blanc.set("facile")
        self.jeton_noir = StringVar()
        self.jeton_noir.set("\u26c2")
        self.jeton_blanc = StringVar()
        self.jeton_blanc.set("\u26c0")

        # Set le font selon la platforme utilisée.
        if platform.system() == "Linux":
            self.font_texte = font.Font(self, ("ubuntu", 20))
        elif platform.system() == "Windows":
            if platform.release() == "10":
                self.font_texte = font.Font(self, ("dax-medium", 20))
            else:
                self.font_texte = font.Font(self, ("constantia", 20))
        else:
            self.font_texte = font.Font(self, ("arial", 20))

        self.font_planche = font.Font(self, ("arialrounded", 14, "bold italic"))
        self.font_coups_possibles = font.Font(self, ("arialrounded", 14))
        self.font_pieces = font.Font(self, ("arialrounded", 40))

        # Set la fenêtre pour qu'elle soit de dimension fixe.
        self.resizable(0, 0)

        # Set le titre de la fenêtre.
        self.title("Othello")

        # Crée notre canvas.
        self.canvas = Canvas(self, width=1233, height=825, background="#f7b9ec")
        self.canvas.pack()

        # Crée des backgrounds
        self.liste_background = [PhotoImage(file="interface/rose.png")]
        self.liste_background.append(PhotoImage(file="interface/briques.png"))
        self.liste_background.append(PhotoImage(file="interface/cubes.png"))
        self.liste_background.append(PhotoImage(file="interface/papier.png"))
        self.liste_background.append(PhotoImage(file="interface/pixels.png"))
        self.background_index = choice(range(5))
        background = self.liste_background[self.background_index]
        self.canvas.create_image(0, 0, image=background, anchor=NW, tags="background")

        # Crée l'image de l'avatar.
        self.image_avatar = PhotoImage(file="interface/avatar.gif")
        self.canvas.create_image(800, 50, image=self.image_avatar, anchor=NW, tags="avatar")

        # Exécute une animation sur un nouveau thread.
        self.image_wink = PhotoImage(file="interface/wink.png")
        self.canvas.create_image(800, 50, image=self.image_wink, anchor=NW, tags="wink", state=HIDDEN)
        threading.Thread(target=self.__wink, daemon=True).start()

        # Crée tout les boutons que nous utiliserons.
        self.__create_widgets()

        # Bind le clic gauche à la méthode __selection_coup_humain().
        self.canvas.bind('<Button-1>', self.__selection_coup_humain)

        # Set le protocole de terminaison.
        self.protocol("WM_DELETE_WINDOW", self.__on_closing)

    def __wink(self):
        """
        Animation pour l'avatar et pour l'astuce. Ceci doit s'exécuter sur un thread séparé.
        """
        while True:
            self.canvas.itemconfig("astuce", state=HIDDEN)
            self.canvas.itemconfig("wink", state=NORMAL)
            time.sleep(0.1)
            self.canvas.itemconfig("wink", state=HIDDEN)
            time.sleep(0.25)
            self.canvas.itemconfig("astuce", state=NORMAL)
            time.sleep(0.25)
            self.canvas.itemconfig("astuce", state=HIDDEN)
            self.canvas.itemconfig("wink", state=NORMAL)
            time.sleep(0.1)
            self.canvas.itemconfig("wink", state=HIDDEN)
            for i in range(8):
                time.sleep(0.25)
                self.canvas.itemconfig("astuce", state=NORMAL)
                time.sleep(0.25)
                self.canvas.itemconfig("astuce", state=HIDDEN)

    def __on_closing(self):
        """
        Affiche un messagebox lorsque l'utilisateur tente de fermer la fenêtre.
        """
        if messagebox.askokcancel("Quitter", "Voulez-vous quitter?"):
            self.destroy()

    def __nouvelle_partie(self):
        """
        Débute l'initialisation d'une nouvelle partie lorsque l'utilisateur clique sur "Nouvelle partie".
        """
        # Attend que les threads ordinateurs finissent leur exécution.
        if self.partie:
            self.partie.terminee = True
            if threading.activeCount() > 2:
                print("Bloqué. Il y a {} threads.".format(str(threading.activeCount())))
                self.after(1000, self.__nouvelle_partie)
                return

            # Détruit la partie précédente.
            self.partie = None
            print("Libre de continuer. Il y a {} threads.\n".format(str(threading.activeCount())))

        # Display les widgets appropriés.
        self.canvas.itemconfig("noir", state=HIDDEN)
        self.canvas.itemconfig("blanc", state=HIDDEN)
        self.canvas.itemconfig("panel", state=HIDDEN)
        self.canvas.itemconfig("dimension", state=NORMAL)
        self.canvas.itemconfig("texte", text="Sélectionnez une\ndimension de jeu")
        self.canvas.delete("case")
        self.canvas.delete("piece")
        self.canvas.delete("compteur")
        self.canvas.delete("focus")
        self.canvas.delete("astuce")

    def __transition_nouvelle_partie(self, nb):
        """
        Exécute lorsque l'utilisateur choisi la dimension de la planche lors de l'initialisation d'une nouvelle partie.
        """
        # Set le nombre de cases et le décalage de la planche.
        self.nb_cases = nb
        self.decalage = int(400 - 35 * nb)

        self.__selection_options_joueur("noir")

    def __selection_options_joueur(self, couleur):
        """
        Permet à l'utilisateur de sélectionner les options du jouer "noir" ou "blanc".
        """
        # Display les widgets appropriés.
        self.canvas.itemconfig("panel", state=NORMAL)
        self.canvas.itemconfig("dimension", state=HIDDEN)
        if couleur == "noir":
            self.canvas.itemconfig("noir", state=NORMAL)
            self.canvas.itemconfig("blanc", state=HIDDEN)
        else:
            self.canvas.itemconfig("blanc", state=NORMAL)
            self.canvas.itemconfig("noir", state=HIDDEN)
        self.canvas.itemconfig("texte", text="Sélectionnez\nles options\ndu joueur {}".format(couleur))

    def __afficher_difficultee(self, couleur, rb_facile, rb_moyen, rb_difficile, b_humain, b_ordinateur):
        """
        Exécute lorsque l'utilisateur sélectionne le bouton "Ordinateur" lors de l'initialisation d'une nouvelle partie.
        """
        # Permet à l'utilisateur de sélectionner une difficultée.
        rb_facile["state"] = "normal"
        rb_moyen["state"] = "normal"
        rb_difficile["state"] = "normal"

        # Soulève le bouton humain.
        b_humain["relief"] = "raised"
        b_humain["background"] = "#b4b4b4"

        # Enfonce le bouton ordinateur.
        b_ordinateur["relief"] = "sunken"
        b_ordinateur["background"] = "#e4e4e4"

        # Set le type du joueur à "ordinateur".
        if couleur == "noir":
            self.type_joueur_noir = "ordinateur"
        else:
            self.type_joueur_blanc = "ordinateur"

    def __desafficher_difficultee(self, couleur, rb_facile, rb_moyen, rb_difficile, b_humain, b_ordinateur):
        """
        Exécute lorsque l'utilisateur sélectionne le bouton "Humain" lors de l'initialisation d'une nouvelle partie.
        """
        # Empêche à l'utilisatuer de sélectionner une difficultée.
        rb_facile["state"] = "disable"
        rb_moyen["state"] = "disable"
        rb_difficile["state"] = "disable"

        # Enfonce le bouton humain.
        b_humain["relief"] = "sunken"
        b_humain["background"] = "#e4e4e4"

        # Soulève le bouton ordinateur.
        b_ordinateur["relief"] = "raised"
        b_ordinateur["background"] = "#b4b4b4"

        # Set le type du joueur à "humain".
        if couleur == "noir":
            self.type_joueur_noir = "humain"
        else:
            self.type_joueur_blanc = "humain"

    def __commencer_partie(self):
        """
        Démarrage d'une nouvelle partie.
        """
        # Display les widgets appropriés.
        self.canvas.itemconfig("blanc", state=HIDDEN)
        self.canvas.itemconfig("noir", state=HIDDEN)
        self.canvas.itemconfig("panel", state=HIDDEN)

        self.canvas.create_text(self.decalage + 70 * self.nb_cases / 2,
                                self.decalage + 70 * (self.nb_cases + 1) - 30,
                                text="", font=self.font_texte, justify=CENTER, tags="compteur")
        self.__dessiner_cases()
        self.__setup_pieces()

        # Création de la partie.
        self.partie = Partie(self, self.nb_cases)
        self.partie.coups_possibles = self.partie.planche.lister_coups_possibles_de_couleur("noir")
        self.update_pieces()

    def __charger_intermediaire(self):
        if messagebox.askokcancel("Charger", "Voulez-vous charger une nouvelle partie?"):
            self.__charger()

    def __charger(self):
        """
        Charge une partie à partir d'un fichier .txt sélectionné par l'utilisateur dans un filedialog.
        La sélection d'un mauvais fichier .txt peut causer des comportements problématiques.
        TODO: vérifier le format du fichier avant de l'exécuter.
        """
        # Attend que les threads ordinateurs finissent leur exécution.
        if self.partie:
            self.partie.terminee = True
            if threading.activeCount() > 2:
                print("Bloqué. Il y a {} threads.".format(str(threading.activeCount())))
                self.after(1000, self.__charger)
                return

            # Détruit la partie précédente.
            self.partie = None
            print("Libre de continuer. Il y a {} threads.\n".format(str(threading.activeCount())))

        # Display les widgets appropriés.
        self.canvas.itemconfig("noir", state=HIDDEN)
        self.canvas.itemconfig("blanc", state=HIDDEN)
        self.canvas.itemconfig("panel", state=HIDDEN)
        self.canvas.itemconfig("dimension", state=HIDDEN)
        self.canvas.delete("case")
        self.canvas.delete("piece")
        self.canvas.delete("compteur")
        self.canvas.delete("focus")
        self.canvas.delete("astuce")

        self.canvas.itemconfig("texte", text="Veuillez choisir\nune sauvegarde")

        file_name = filedialog.askopenfilename(title="Veuillez choisir une sauvegarde",
                                               filetypes=[('fichier texte', '.txt')])

        self.partie = Partie(self, nom_de_fichier=file_name)

        # Set le nombre de cases et le décalage de la planche.
        self.nb_cases = self.partie.planche.nb_cases
        self.decalage = int(400 - 35 * self.nb_cases)

        self.canvas.create_text(self.decalage + 70 * self.nb_cases / 2,
                                self.decalage + 70 * (self.nb_cases + 1) - 30,
                                text="", font=self.font_texte, justify=CENTER, tags="compteur")
        self.partie.planche.calculer_nombres_pieces()
        self.__dessiner_cases()
        self.__setup_pieces()
        self.update_pieces()

        # Vérifie si la partie est terminée.
        if self.partie.partie_terminee():
            return

        # Vérifie si le joueur doit passer son tour.
        if not self.partie.coups_possibles:
            self.partie.passer_tour()

    def __create_taskbar(self):
        """
        Crée le taskbar de l'application.
        """
        menubar = Menu(self)
        menubar.add_command(label="Nouvelle partie", command=self.__nouvelle_partie)
        menubar.add_command(label="Charger", command=self.__charger_intermediaire)
        menubar.add_command(label="Astuce", command=self.__astuce)
        menubar.add_command(label="\u23EA Retour", command=self.__retour)
        menubar.add_command(label="Arrière-plan", command=self.__change_background)
        menubar.add_command(label="Règles", command=self.__regles)
        menubar.add_command(label="Crédits", command=self.__credits)
        menubar.add_command(label="Quitter", command=self.__on_closing)
        self.config(menu=menubar)

    def __change_background(self):
        self.background_index += 1
        if self.background_index == 5:
            self.background_index = 0
        self.canvas.itemconfig("background", image=self.liste_background[self.background_index])

    def __retour(self):
        """
        Permet au joueur humain, lors de son tour, de retourner en arrière. Ceci utilise la pile des coups joués.
        """
        if self.partie and self.partie.joueur_courant.obtenir_type_joueur() == "humain" and not self.partie.terminee:
            self.partie.lock = True

            # Retourne si historique vide.
            if len(self.partie.planche.historique_coups_jouees) < 2:
                return

            deuxieme_coup = self.partie.planche.historique_coups_jouees[-2]

            if len(self.partie.planche.historique_coups_jouees) == 2 and deuxieme_coup[0] is None:
                return

            self.partie.planche.restaurer_cases()
            self.partie.planche.restaurer_cases()

            self.update_pieces()
            self.partie.planche.calculer_nombres_pieces()
            self.set_avatar_texte("Vous êtes\nrevenus\nsur vos pas")
            self.move_focus(-5, 0)
            self.move_astuce(-5, 0)

            if deuxieme_coup[0] is None:
                self.__retour()

            self.partie.lock = False

    def __astuce(self):
        """
        Affiche un astuce au joueur en employant l'algorithme minimax.
        """
        if not self.partie or self.partie.joueur_courant.obtenir_type_joueur() != "humain":
            return

        pos = self.partie.joueur_courant.choisir_coup_minimax(self.partie.coups_possibles, 4)
        self.move_astuce(pos[0], pos[1])

        self.play_sound("astuce")

    @staticmethod
    def __regles():
        """
        Affiche les règles du jeu.
        """
        f = open("regles.txt", "r", encoding='utf-8')
        texte = f.read()
        messagebox.showinfo("Règles du jeu", texte)
        f.close()

    @staticmethod
    def __credits():
        """
        Affiche les crédits.
        """
        f = open("credits.txt", "r", encoding='utf-8')
        texte = f.read()
        messagebox.showinfo("Crédits", texte)
        f.close()

    def __selection_coup_humain(self, event):
        """
        Permet au joueur humain de cliquer sur une case de jeu pour placer une pièce.

        Args:
            event: un mouse event avec des coordonnées x et y.
        """
        if not self.partie or self.partie.joueur_courant.obtenir_type_joueur() != "humain" or self.partie.terminee:
            return

        # Joue un coup à la case sélectionnée.
        event.x -= self.decalage
        event.y -= self.decalage
        ligne = event.y // 70
        colonne = event.x // 70
        self.partie.lock = True
        self.partie.tour_humain((ligne, colonne))
        self.partie.lock = False

    def play_sound(self, type_de_son):
        """
        Joue un son audio lorsqu'une pièce est jouée sur la planche.
        """
        if type_de_son == "gamepiece":
            self.gamepiece_sound.play()
        elif type_de_son == "astuce":
            self.astuce_sound.play()
        elif type_de_son == "winner":
            self.winner_sound.play()
        elif type_de_son == "loser":
            self.loser_sound.play()

    def set_avatar_texte(self, texte):
        """
        Setter pour le text widget qui affiche un message à l'utilisateur.

        Args:
            texte: un string qui devrait être affiché.
        """
        self.canvas.itemconfig("texte", text=texte)

    def set_compteur_texte(self, texte):
        """
        Setter pour le text widget qui affiche le compteur de pièces noires et blanches.

        Args:
            texte: un string contenant le nombre de pièces noires et blanches.
        """
        self.canvas.itemconfig("compteur", text=texte)

    @staticmethod
    def avertissement(exception):
        """
        Affiche un avertissement lorsqu'un coup tenté est invalide.

        Args:
            exception: le message d'exception à afficher.
        """
        messagebox.showerror("Position Invalide", exception)

    def __dessiner_cases(self):
        """
        Dessine la planche de jeu.
        """
        for i in range(-1, self.nb_cases):
            for j in range(-1, self.nb_cases):

                # Dessine une coordonnée sur l'axe vertical.
                if i == -1:
                    if j == -1:
                        continue
                    self.canvas.create_text(30 + self.decalage + 70 * j, self.decalage + 55 + 70 * i,
                                            text=str(j + 1), justify=CENTER, tags="case",
                                            state=DISABLED, font=self.font_planche)

                # Dessine une coordonnée sur l'axe horizontal.
                elif j == -1:
                    self.canvas.create_text(55 + self.decalage + 70 * j, 30 + self.decalage + 70 * i,
                                            text=str(i + 1), justify=CENTER, tags="case",
                                            state=DISABLED, font=self.font_planche)

                # Dessine un rectangle représentant une case de jeu.
                else:
                    self.canvas.create_rectangle(self.decalage + 70 * j, self.decalage + 70 * i,
                                                 self.decalage + 70 * (j + 1), self.decalage + 70 * (i + 1),
                                                 fill="#63b76c", tags='case', width=3, activewidth=5,
                                                 activefill="#83d78c")

        # Crée un rectangle pour l'option astuce.
        self.canvas.create_rectangle(0, 0, 0, 0, fill="#f7f454", tags='astuce', width=3)

        # Crée un rectangle qui garde en focus le dernier coup joué.
        self.canvas.create_rectangle(0, 0, 0, 0, fill="#63b76c", tags='focus', width=6)

    def move_focus(self, x, y):
        """
        Change la position du rectangle de focus lorsqu'un coup est joué.

        x: le numéro de la rangée de la planche.
        y: le numéro de la colonne de la planche.
        """
        self.canvas.coords("focus", self.decalage + 70 * y, self.decalage + 70 * x,
                           self.decalage + 70 * (y + 1), self.decalage + 70 * (x + 1))

    def move_astuce(self, x, y):
        """
        Change la position du rectangle de l'astuce.

        Args:
            x: le numéro de la rangée de la planche.
            y: le numéro de la colonne de la planche.
        """
        self.canvas.coords("astuce", self.decalage + 70 * y, self.decalage + 70 * x,
                           self.decalage + 70 * (y + 1), self.decalage + 70 * (x + 1))

    def __setup_pieces(self):
        """
        Crée une zone de texte pour chaque case de jeu. Elles sont stockées dans un dictionnaire.
        """
        for i in range(self.nb_cases):
            for j in range(self.nb_cases):
                self.pieces[i, j] = self.canvas.create_text(self.decalage + 35 + 70 * j, self.decalage + 35 + 70 * i,
                                                            text="", justify=CENTER, tags="piece",
                                                            state=DISABLED, font=self.font_pieces)

    def update_pieces(self):
        """
        Update les zones de texte associées à chaque case de jeu.
        """
        # Pour chaque case.
        for i in range(self.nb_cases):
            for j in range(self.nb_cases):
                # Si une pièce occupe la case.
                if (i, j) in self.partie.planche.cases:
                    # Sélectionne le texte selon la couleur de la pièce.
                    if self.partie.planche.cases[i, j].est_blanc():
                        texte = str(self.jeton_blanc.get())
                    else:
                        texte = self.jeton_noir.get()
                    # Dessine la pièce.
                    self.canvas.itemconfig(self.pieces[i, j], text=texte, font=self.font_pieces)
                # Dessine un compteur indiquant le nombre potentiel de pièces mangées à une case.
                elif (i, j) in self.partie.planche.nombres_pieces_mangees_par_case:
                    texte = str(self.partie.planche.nombres_pieces_mangees_par_case[i, j])
                    self.canvas.itemconfig(self.pieces[i, j], text=texte, font=self.font_coups_possibles)
                else:
                    self.canvas.itemconfig(self.pieces[i, j], text="")

    def __create_widgets(self):
        """
        Crée plusieurs boutons et radiobuttons qui seront utilisés lors de l'initialisation d'une nouvelle partie.
        """
        # ==============================================================================================================
        # 3 caractères unicode pour les jetons noirs et 3 pour les jetons blanc.
        jeton_noir_a = "\u26c2"
        jeton_noir_b = "\u2665"
        jeton_noir_c = "\u265e"
        jeton_blanc_a = "\u26c0"
        jeton_blanc_b = "\u2661"
        jeton_blanc_c = "\u2658"

        # ==============================================================================================================
        # Crée une zone de texte.
        self.canvas.create_text(970, 330,
                                text="Bienvenue!\nCliquez sur\n'Nouvelle partie'\npour commencer\nà jouer",
                                font=self.font_texte, justify=CENTER, tags="texte")

        # ==============================================================================================================
        # Crée un rectangle pour les options.
        self.canvas.create_rectangle(30, 125, 715, 500, fill="#92b2c3", width=5, tags="panel", state=HIDDEN)

        # ==============================================================================================================
        # Crée trois boutons pour les dimensions de planche.
        b_6x6 = Button(self, text="6 x 6", command=lambda: self.__transition_nouvelle_partie(6))
        b_6x6.configure(width=25, height=3, background="#b4b4b4", font=self.font_texte, bd=5)
        self.canvas.create_window(220, 150, anchor=NW, window=b_6x6, tags="dimension", state=HIDDEN)

        b_8x8 = Button(self, text="8 x 8", command=lambda: self.__transition_nouvelle_partie(8))
        b_8x8.configure(width=25, height=3, background="#b4b4b4", font=self.font_texte, bd=5)
        self.canvas.create_window(220, 300, anchor=NW, window=b_8x8, tags="dimension", state=HIDDEN)

        b_10x10 = Button(self, text="10 x 10", command=lambda: self.__transition_nouvelle_partie(10))
        b_10x10.configure(width=25, height=3, background="#b4b4b4", font=self.font_texte, bd=5)
        self.canvas.create_window(220, 450, anchor=NW, window=b_10x10, tags="dimension", state=HIDDEN)

        # ==============================================================================================================
        # Crée des boutons pour la difficultée du joueur noir.
        rb_facile_n = Radiobutton(self, text="Facile", value="facile",
                                  highlightthickness=0, variable=self.difficultee_noir,
                                  font=self.font_texte, bg="#92b2c3", state=DISABLED, activebackground="#92b2c3")
        self.canvas.create_window(490, 250, anchor=NW, window=rb_facile_n, tags="noir", state=HIDDEN)

        rb_moyen_n = Radiobutton(self, text="Moyen", value="moyen",
                                 highlightthickness=0, variable=self.difficultee_noir,
                                 font=self.font_texte, bg="#92b2c3", state=DISABLED, activebackground="#92b2c3")
        self.canvas.create_window(490, 300, anchor=NW, window=rb_moyen_n, tags="noir", state=HIDDEN)

        rb_difficile_n = Radiobutton(self, text="Difficile", value="difficile",
                                     highlightthickness=0, variable=self.difficultee_noir,
                                     font=self.font_texte, bg="#92b2c3", state=DISABLED, activebackground="#92b2c3")
        self.canvas.create_window(490, 350, anchor=NW, window=rb_difficile_n, tags="noir", state=HIDDEN)

        # ==============================================================================================================
        # Crée des boutons pour la difficultée due joueur blanc.
        rb_facile_b = Radiobutton(self, text="Facile", value="facile",
                                  highlightthickness=0, variable=self.difficultee_blanc,
                                  font=self.font_texte, bg="#92b2c3", state=DISABLED, activebackground="#92b2c3")
        self.canvas.create_window(490, 250, anchor=NW, window=rb_facile_b, tags="blanc", state=HIDDEN)

        rb_moyen_b = Radiobutton(self, text="Moyen", value="moyen",
                                 highlightthickness=0, variable=self.difficultee_blanc,
                                 font=self.font_texte, bg="#92b2c3", state=DISABLED, activebackground="#92b2c3")
        self.canvas.create_window(490, 300, anchor=NW, window=rb_moyen_b, tags="blanc", state=HIDDEN)

        rb_difficile_b = Radiobutton(self, text="Difficile", value="difficile",
                                     highlightthickness=0, variable=self.difficultee_blanc,
                                     font=self.font_texte, bg="#92b2c3", state=DISABLED, activebackground="#92b2c3")
        self.canvas.create_window(490, 350, anchor=NW, window=rb_difficile_b, tags="blanc", state=HIDDEN)

        # ==============================================================================================================
        # Crée des boutons pour le type du joueur noir.
        b_humain_n = Button(self, text="Humain", bd=5,
                            command=lambda: self.__desafficher_difficultee("noir", rb_facile_n, rb_moyen_n,
                                                                           rb_difficile_n, b_humain_n, b_ordinateur_n))
        b_humain_n.configure(width=15, height=2, background="#b4b4b4", font=self.font_texte)
        self.canvas.create_window(50, 150, anchor=NW, window=b_humain_n, tags="noir", state=HIDDEN)

        b_ordinateur_n = Button(self, text="Ordinateur", bd=5,
                                command=lambda: self.__afficher_difficultee("noir", rb_facile_n, rb_moyen_n,
                                                                            rb_difficile_n, b_humain_n, b_ordinateur_n))
        b_ordinateur_n.configure(width=15, height=2, background="#b4b4b4", font=self.font_texte)
        self.canvas.create_window(700, 150, anchor=NE, window=b_ordinateur_n, tags="noir", state=HIDDEN)

        # ==============================================================================================================
        # Crée des boutons pour le type du joueur blanc.
        b_humain_b = Button(self, text="Humain", bd=5,
                            command=lambda: self.__desafficher_difficultee("blanc", rb_facile_b, rb_moyen_b,
                                                                           rb_difficile_b, b_humain_b, b_ordinateur_b))
        b_humain_b.configure(width=15, height=2, background="#b4b4b4", font=self.font_texte)
        self.canvas.create_window(50, 150, anchor=NW, window=b_humain_b, tags="blanc", state=HIDDEN)

        b_ordinateur_b = Button(self, text="Ordinateur", bd=5,
                                command=lambda: self.__afficher_difficultee("blanc", rb_facile_b, rb_moyen_b,
                                                                            rb_difficile_b, b_humain_b, b_ordinateur_b))
        b_ordinateur_b.configure(width=15, height=2, background="#b4b4b4", font=self.font_texte)
        self.canvas.create_window(700, 150, anchor=NE, window=b_ordinateur_b, tags="blanc", state=HIDDEN)

        # ==============================================================================================================
        # Crée des boutons pour les jetons noirs.
        rb_jeton_1 = Radiobutton(self, text=jeton_noir_a, variable=self.jeton_noir, value=jeton_noir_a,
                                 highlightthickness=0, font=self.font_pieces, activebackground="#92b2c3",
                                 background="#92b2c3", borderwidth=3)
        self.canvas.create_window(120, 400, anchor=NW, window=rb_jeton_1, tags="noir", state=HIDDEN)

        rb_jeton_2 = Radiobutton(self, text=jeton_noir_b, variable=self.jeton_noir, value=jeton_noir_b,
                                 highlightthickness=0, font=self.font_pieces, activebackground="#92b2c3",
                                 background="#92b2c3", borderwidth=3)
        self.canvas.create_window(320, 400, anchor=NW, window=rb_jeton_2, tags="noir", state=HIDDEN)

        rb_jeton_3 = Radiobutton(self, text=jeton_noir_c, variable=self.jeton_noir, value=jeton_noir_c,
                                 highlightthickness=0, font=self.font_pieces, activebackground="#92b2c3",
                                 background="#92b2c3", borderwidth=3)
        self.canvas.create_window(520, 400, anchor=NW, window=rb_jeton_3, tags="noir", state=HIDDEN)

        # ==============================================================================================================
        # Crée des boutons pour les jetons blancs.
        rb_jeton_1 = Radiobutton(self, text=jeton_blanc_a, variable=self.jeton_blanc, value=jeton_blanc_a,
                                 highlightthickness=0, font=self.font_pieces, activebackground="#92b2c3",
                                 background="#92b2c3", borderwidth=3)
        self.canvas.create_window(120, 400, anchor=NW, window=rb_jeton_1, tags="blanc", state=HIDDEN)

        rb_jeton_2 = Radiobutton(self, text=jeton_blanc_b, variable=self.jeton_blanc, value=jeton_blanc_b,
                                 highlightthickness=0, font=self.font_pieces, activebackground="#92b2c3",
                                 background="#92b2c3", borderwidth=3)
        self.canvas.create_window(320, 400, anchor=NW, window=rb_jeton_2, tags="blanc", state=HIDDEN)

        rb_jeton_3 = Radiobutton(self, text=jeton_blanc_c, variable=self.jeton_blanc, value=jeton_blanc_c,
                                 highlightthickness=0, font=self.font_pieces, activebackground="#92b2c3",
                                 background="#92b2c3", borderwidth=3)
        self.canvas.create_window(520, 400, anchor=NW, window=rb_jeton_3, tags="blanc", state=HIDDEN)

        # ==============================================================================================================
        # Crée un bouton "Suivant" pour le joueur noir.
        b_suivant = Button(self, text="Suivant", command=lambda: self.__selection_options_joueur("blanc"))
        b_suivant.configure(width=15, height=1, background="#b4b4b4", font=self.font_texte, bd=5)
        self.canvas.create_window(418, 545, anchor=NW, window=b_suivant, tags="noir", state=HIDDEN)

        # ==============================================================================================================
        # Crée un bouton "Suivant" pour le joueur blanc.
        b_suivant = Button(self, text="Suivant", command=self.__commencer_partie)
        b_suivant.configure(width=15, height=1, background="#b4b4b4", font=self.font_texte, bd=5)
        self.canvas.create_window(418, 545, anchor=NW, window=b_suivant, tags="blanc", state=HIDDEN)

        # ==============================================================================================================
        # Sélectionne l'option "Humain" par défault.
        self.__desafficher_difficultee("noir", rb_facile_n, rb_moyen_n, rb_difficile_n, b_humain_n, b_ordinateur_n)
        self.__desafficher_difficultee("blanc", rb_facile_b, rb_moyen_b, rb_difficile_b, b_humain_b, b_ordinateur_b)
