from random import choice
import threading
import time
from othello.simulateur import Simulateur


class Joueur:
    """
    Classe générale de joueur.
    """
    def __init__(self, couleur, partie):
        """
        Le constructeur global de Joueur.

        Args:
            couleur: la couleur qui sera jouée par le joueur.
            partie: une référence à la partie dans laquelle le joueur existe.
        """
        assert couleur in ["blanc", "noir"], "Piece: couleur invalide."
        self.couleur = couleur
        self.partie = partie
        self.simulateur = Simulateur(self.couleur, partie.planche)

    def obtenir_type_joueur(self):
        """
        Cette méthode sera utilisée par les sous-classes JoueurHumain et JoueurOrdinateur.

        Returns:
            Le type de joueur, 'ordinateur' ou 'humain'.
        """
        pass

    def choisir_coup_minimax(self, coups_possibles, profondeur):
        """
        Séléctionne un coup parmis les coups possibles.
        Emploie un minimax algorithme avec une évaluation sur tableau.

        Args:
            coups_possibles: La liste des coups possibles.
            profondeur: La profondeur de l'arbe minimax.

        Returns:
             un couple (ligne, colonne) représentant la position du coup désiré.
        """
        # Liste qui stock un pointage pour chacun des coups possibles
        resultats_simulations = []

        # Pour chaque coup possible, exécute une simulation de jeu pour y attribuer un pointage.
        for coup in coups_possibles:
            resultats_simulations.append(self.simulateur.minimax(coup, False, profondeur, -3000, 3000))

        # Séléctionne une position par rapport à la valeur maximale des résultats de simulations.
        maximum = max(resultats_simulations)
        position = choice([coups_possibles[i] for i, e in enumerate(resultats_simulations) if e == maximum])

        return position


class JoueurHumain(Joueur):
    """
    Classe modélisant un joueur humain.
    """
    def __init__(self, couleur, partie):
        super().__init__(couleur, partie)

    def obtenir_type_joueur(self):
        return "humain"


class JoueurOrdinateur(Joueur):
    """
    Classe modélisant un joueur Ordinateur.

    Il utilise un de trois algorithmes pour choisir son coup:
        - facile: l'ordinateur joue aléatoirement.
        - moyen: l'ordinateur emploie un minimax algorithme (profondeur 2) avec une évaluation sur tableau.
        - difficile: l'ordinateur emploie un minimax algorithme (profondeur 4) avec une évaluation sur tableau.
    """
    def __init__(self, couleur, partie, difficultee):
        """
        Cette méthode va construire un objet Joueur Ordinateur.

        Args:
            couleur: la couleur qui sera jouée par le joueur.
            partie: un pointeur vers la partie.
            difficultee: "facile", "moyen" ou "difficile".
        """
        super().__init__(couleur, partie)
        self.difficultee = difficultee
        threading.Thread(target=self.loop_ordinateur, daemon=True).start()

    def obtenir_type_joueur(self):
        return "ordinateur"

    def loop_ordinateur(self):
        """
        Le loop de l'ordinateur qui joue sur un thread personnel à lui.
        """
        while True:
            # Si la partie est terminée le thread peut conclure son exécution.
            if self.partie.terminee:
                print("Thread [{}] terminé.".format(self.couleur))
                return

            # L'ordinateur doit attendre que c'est son tour.
            if self.partie.couleur_joueur_courant == self.couleur and not self.partie.lock:

                # Set un flag pour dire que l'ordinateur est en train d'exécuter.
                self.partie.lock = True
                time.sleep(0.5)

                # L'ordinateur choisi un coup et joue.
                self.choisir_coup()

                # Reset le flag pour dire que l'ordinateur à fini d'exécuter.
                self.partie.lock = False
                time.sleep(0.5)
            time.sleep(1)

    def choisir_coup(self):
        """
        L'ordinateur choisi son coup en appelant la méthode correspondante selon son niveau de difficulté.
        """
        # Return (-1, -1) par défaut pour signaler une erreure.
        position = (-1, -1)

        coups_possibles = self.partie.coups_possibles

        if not coups_possibles:
            return

        if self.partie.terminee:
            return

        if self.difficultee == "facile":
            position = self.choisir_coup_aleatoire(coups_possibles)

        elif self.difficultee == "moyen":
            position = self.choisir_coup_minimax(coups_possibles, 2)

        elif self.difficultee == "difficile":
            position = self.choisir_coup_minimax(coups_possibles, 4)

        message = "L'ordinateur {}\njoue à la ligne\n{} et à la colonne {}".format(self.couleur,
                                                                                   position[0] + 1,
                                                                                   position[1] + 1)

        self.partie.fenetre.set_avatar_texte(message)
        self.partie.jouer_tour(position)
        self.partie.fenetre.update_pieces()

    @staticmethod
    def choisir_coup_aleatoire(coups_possibles):
        """
        Séléctionne un coup aléatoirement parmis tous les coups possibles.

        Args:
            coups_possibles: La liste des coups possibles.

        Returns:
            un couple (ligne, colonne) représentant la position du coup désiré.
        """
        position = choice(coups_possibles)
        return position
