from othello.planche import Planche
from othello.joueur import JoueurOrdinateur, JoueurHumain
from othello.exceptions import ErreurPositionCoup
from tkinter import messagebox


class Partie:
    def __init__(self, fenetre, nb_cases=None, nom_de_fichier=None):
        """
        Méthode d'initialisation d'une partie.

        Args:
            nb_cases: le nombre de cases voulu pour une nouvelle partie.
            nom_de_fichier: le path d'un fichier pour charger une partie déjà commencée.
        """
        # La fenêtre graphique du jeu.
        self.fenetre = fenetre

        # Le tour précédent a-t-il été passé?
        self.tour_precedent_passe = False

        # Les deux tours précédents ont-ils étés passés?
        self.deux_tours_passes = False

        # Liste des coups possibles ce tour.
        self.coups_possibles = []

        # Le joueur de couleur blanc.
        self.joueur_blanc = None

        # Le jouer de couleur noir.
        self.joueur_noir = None

        # Le joueur à qui est le tour.
        self.joueur_courant = None

        # Couleur du joueur à qui est le tour.
        self.couleur_joueur_courant = "noir"

        # Un lock nous permet de nous assurer qu'un seul joueur puisse joueur à la fois.
        self.lock = False

        # Boolean: la partie est-elle terminée?
        self.terminee = False

        # Planche du jeu.
        self.planche = None

        # Si un nombre de cases à été spécifié crée une nouvelle partie.
        if nb_cases:
            # Set le message sur la fenêtre.
            self.fenetre.set_avatar_texte("C'est au joueur\nnoir à jouer")

            # Planche du jeu.
            self.planche = Planche(self, nb_cases)

            # Initialise les deux joueurs avec les informations recueillies par la fenêtre.
            self.__initialiser_joueurs(fenetre.type_joueur_noir, fenetre.type_joueur_blanc,
                                       fenetre.difficultee_noir.get(), fenetre.difficultee_blanc.get())

        # Si un nom de fichier à été spécifié charge une partie à partir de la sauvegarde.
        else:
            self.charger(nom_de_fichier)
            self.fenetre.set_avatar_texte("Sauvegarde\nchargée!")

    def __initialiser_joueurs(self, type_noir, type_blanc, difficulte_noir, difficulte_blanc):
        """
        On initialise ici trois attributs : joueur_noir, joueur_noir et joueur_courant (initialisé à joueur_noir).
        """
        # Initialise le joueur noir.
        self.joueur_noir = self.__creer_joueur(type_noir, "noir", difficulte_noir)

        # Initialise le joueur blanc.
        self.joueur_blanc = self.__creer_joueur(type_blanc, "blanc", difficulte_blanc)

        # Le joueur noir joue en premier.
        self.joueur_courant = self.joueur_noir

    def __creer_joueur(self, type_joueur, couleur, difficulte=None):
        """
        Crée l'objet Joueur approprié, selon le type passé en paramètre.

        Args:
            type_joueur: le type de joueur, "ordinateur" ou "humain".
            couleur: la couleur du pion joué par le jouer, "blanc" ou "noir".
            difficulte: "facile", "moyen" our "difficile".

        Returns:
            Un objet JoueurHumain si le type est "Humain", JoueurOrdinateur sinon.
        """
        # Crée un joueur humain.
        if type_joueur == "humain":
            return JoueurHumain(couleur, self)

        if type_joueur == "ordinateur":
            # Crée un joueur ordinateur facile.
            if difficulte == "facile":
                return JoueurOrdinateur(couleur, self, "facile")

            # Crée un joueur ordinateur moyen.
            if difficulte == "moyen":
                return JoueurOrdinateur(couleur, self, "moyen")

            # Crée un joueur ordinateur difficile.
            if difficulte == "difficile":
                return JoueurOrdinateur(couleur, self, "difficile")

    def __valider_position_coup(self, position_coup):
        """
        Vérifie la validité de la position désirée pour le coup. On retourne un message d'erreur approprié pour
        chacune des trois situations suivantes :

        1) Le coup tenté ne représente pas une position valide de la planche de jeu.
        2) Une pièce se trouve déjà à la position souhaitée.
        3) Le coup ne fait pas partie de la liste des coups valides.

        Args:
            position_coup: La position du coup à valider.

        Returns:
            True si valide, False ou une exception sinon.
        """
        # Position en dehors de la planche.
        if not self.planche.position_valide(position_coup):
            return False

        # Position déjà occupée.
        if self.planche.get_piece(position_coup) is not None:
            raise ErreurPositionCoup("Une pièce occupe déjà la position désirée.")

        # Coup non possible.
        if position_coup not in self.coups_possibles:
            raise ErreurPositionCoup("Le coup n'est pas possible.")

        return True

    def jouer_tour(self, position):
        """
        Joue un tour pour le joueur actuel.

        Args:
            position: la position qui devrait être jouée.
        """
        self.fenetre.play_sound("gamepiece")
        self.fenetre.move_focus(position[0], position[1])
        self.planche.jouer_coup(position, self.couleur_joueur_courant)
        self.tour_precedent_passe = False
        self.deux_tours_passes = False
        self.__intermediaire()

    def __intermediaire(self):
        """
        Fonction qui exécute entre chaque tour du jeu.
        """
        # Change le joueur courant à l'autre.
        if self.couleur_joueur_courant == "noir":
            self.couleur_joueur_courant = "blanc"
            self.joueur_courant = self.joueur_blanc
        else:
            self.couleur_joueur_courant = "noir"
            self.joueur_courant = self.joueur_noir

        # Update la liste des coups possibles.
        self.coups_possibles = self.planche.lister_coups_possibles_de_couleur(self.couleur_joueur_courant)

        # Update le compteur du nombre de pièces.
        self.planche.calculer_nombres_pieces()

        # Vérifie si la partie est terminée.
        if self.partie_terminee():
            return

        # Vérifie si le joueur doit passer son tour.
        if not self.coups_possibles:
            self.passer_tour()

    def tour_humain(self, position):
        """
        Excéute le tour d'un joueur humain qui tente de joueur un coup.

        Args:
            position: position du coup tenté par le joueur humain.
        """
        # Si le coup est valide le joueur peut le jouer.
        try:
            validation = self.__valider_position_coup(position)
        except ErreurPositionCoup as e:
            self.fenetre.avertissement(e)
        else:
            if validation:
                self.fenetre.move_astuce(-5, 0)
                self.jouer_tour(position)
                if not self.terminee:
                    self.fenetre.set_avatar_texte("C'est au joueur\n{} à jouer".format(self.couleur_joueur_courant))
                self.fenetre.update_pieces()

    def passer_tour(self):
        """
        Affiche un message indiquant que le joueur de la couleur courante ne peut jouer avec l'état actuel de la
        planche et qu'il doit donc passer son tour.
        """
        self.fenetre.update_pieces()

        self.planche.historique_coups_jouees.append((None, None))

        if self.joueur_courant.obtenir_type_joueur() == "humain":
            messagebox.showinfo("Tour passé", "Aucun coup possible pour le JOUEUR {},"
                                              " le tour est passé.".format(self.couleur_joueur_courant.upper()))
        else:
            messagebox.showinfo("Tour passé", "Aucun coup possible pour l'ORDINATEUR {},"
                                              " le tour est passé.".format(self.couleur_joueur_courant.upper()))

        # Si le dernier tour à déjà été passé, les deux derniers tours ont donc été passés.
        if self.tour_precedent_passe:
            self.deux_tours_passes = True
        else:
            self.tour_precedent_passe = True

        self.__intermediaire()
        self.fenetre.update_pieces()

    def partie_terminee(self):
        """
        Détermine si la partie est terminée, Une partie est terminée si toutes les cases de la planche sont remplies
        ou si deux tours consécutifs ont été passés ou si un joueur n'a plus aucune pièces.
        """
        if self.deux_tours_passes or self.planche.nb_pieces == self.planche.nb_cases**2 \
                or self.planche.nb_pieces_blanc == 0 or self.planche.nb_pieces_noir == 0:

            self.terminee = True
            self.couleur_joueur_courant = None
            # self.lock = False
            self.__determiner_gagnant()
            return True
        else:
            return False

    def __determiner_gagnant(self):
        """
        Détermine le gagnant de la partie. Le gagnant est simplement la couleur pour laquelle il y a le plus de
        pions sur la planche de jeu.
        """
        # Joueur blanc remporte la partie.
        if self.planche.nb_pieces_blanc > self.planche.nb_pieces_noir:
            gagnant = self.joueur_blanc
            perdant = self.joueur_noir

        # Joueur noir remporte la partie.
        elif self.planche.nb_pieces_blanc < self.planche.nb_pieces_noir:
            perdant = self.joueur_blanc
            gagnant = self.joueur_noir

        # Sinon le match est nul.
        else:
            message = "MATCH NUL,\nles deux joueurs\nfinissent avec\n{} pièces!".format(self.planche.nb_pieces_noir)
            self.fenetre.set_avatar_texte(message)
            return

        if gagnant.obtenir_type_joueur() == "ordinateur" and perdant.obtenir_type_joueur() == "humain":
            message = "Meilleure\nchance la\nprochaine fois!"
            self.fenetre.play_sound("loser")

        else:
            message = "Joueur {}\nremporte,\nFÉLICITATIONS!".format(gagnant.couleur)
            self.fenetre.play_sound("winner")

        self.fenetre.set_avatar_texte(message)

    def sauvegarder(self, nom_fichier):
        """
        Sauvegarde une partie dans un fichier. Le fichier condiendra:
        - Une ligne indiquant la couleur du joueur courant.
        - Une ligne contenant True ou False, si le tour précédent a été passé.
        - Une ligne contenant True ou False, si les deux derniers tours ont été passés.
        - Une ligne contenant le type du joueur blanc.
        - Une ligne contenant le type du joueur noir.
        - Le reste des lignes correspondant à la planche. Voir la méthode convertir_en_chaine de la planche
         pour le format.

        ATTENTION : L'ORDRE DES PARAMÈTRES SAUVEGARDÉS EST OBLIGATOIRE À RESPECTER.
                    Des tests automatiques seront roulés lors de la correction et ils prennent pour acquis que le
                    format plus haut est respecté. Vous perdrez des points si vous dérogez du format.

        Args:
            nom_fichier: Le nom du fichier où sauvegarder, un string.
        """

        # Ouverture d'un fichier sous la mode write.
        fichier = open(nom_fichier, 'w')

        # Write la couleur du joueur courant.
        fichier.write(self.couleur_joueur_courant + "\n")

        # Write True si le tour précédent à été passé, False sinon.
        fichier.write(str(self.tour_precedent_passe) + "\n")

        # Write True si les deux tours précédents ont étés passés, False sinon.
        fichier.write(str(self.deux_tours_passes) + "\n")

        # Write le type du joueur blanc.
        fichier.write(self.joueur_blanc.obtenir_type_joueur() + "\n")

        # Write le type du joueur noir.
        fichier.write(self.joueur_noir.obtenir_type_joueur() + "\n")

        # Write l'état de la planche.
        fichier.write(self.planche.convertir_en_chaine())

        # Fermeture du fichier.
        fichier.close()

    def charger(self, nom_fichier):
        """
        Charge une partie dans à partir d'un fichier. Le fichier a le même format que la méthode de sauvegarde.

        Args:
            nom_fichier: Le nom du fichier à charger, un string.
        """

        self.planche = Planche(self, 8)

        # Ouverture d'un fichier sous la mode read.
        fichier = open(nom_fichier, 'r')

        # Read la couleur du joueur courant.
        self.couleur_joueur_courant = fichier.readline().rstrip()

        # Read si le tour précédent à été passé.
        if fichier.readline().rstrip() == "False":
            self.tour_precedent_passe = False
        else:
            self.tour_precedent_passe = True

        # Read si les deux tours précédents ont étés passés.
        if fichier.readline().rstrip() == "False":
            self.deux_tours_passes = False
        else:
            self.deux_tours_passes = True

        # Read le type du joueur blanc.
        if fichier.readline().lower().rstrip() == "humain":
            self.joueur_blanc = self.__creer_joueur("humain", "blanc")
        else:
            self.joueur_blanc = self.__creer_joueur("ordinateur", "blanc", "moyen")

        # Read le type du joueur noir.
        if fichier.readline().lower().rstrip() == "humain":
            self.joueur_noir = self.__creer_joueur("humain", "noir")
        else:
            self.joueur_noir = self.__creer_joueur("ordinateur", "noir", "moyen")

        # Set le joueur courant selon la variable couleur_joueur_courant.
        if self.couleur_joueur_courant == "blanc":
            self.joueur_courant = self.joueur_blanc
        else:
            self.joueur_courant = self.joueur_noir

        # Set le board selon le restant des lignes du fichier.
        self.planche.charger_dune_chaine(fichier.readlines())

        # Fermeture du fichier.
        fichier.close()

        # Update la liste des coups possibles.
        self.coups_possibles = self.planche.lister_coups_possibles_de_couleur(self.couleur_joueur_courant)
