from othello.piece import Piece


class Planche:
    """
    Classe représentant la planche d'un jeu d'Othello.
    """
    def __init__(self, partie, nb_cases):
        """
        Méthode initialisant une nouvelle planche.

        Args:
            partie: un pointeur vers la partie.
            nb_cases: la dimension de la planche.
        """
        # Store la partie dans laquelle la planche existe.
        self.partie = partie

        # Dictionnaire de cases. La clé est une position (ligne, colonne), et la valeur une instance de la classe Piece.
        self.cases = {}

        # On joue au Othello nb_cases x nb_cases.
        self.nb_cases = nb_cases

        # Liste qui store les positions qui ont été mangées le dernier coup (utilisé par l'ordinateur).
        self.positions_mangees_dernier_coup = []

        # Dictionnaire qui store le nombre potentiel de pieces mangées par case pour le tour courant.
        self.nombres_pieces_mangees_par_case = {}

        # Variables qui gardent à jour le nombre de pièces noires, blanches et leur total.
        self.nb_pieces = 0
        self.nb_pieces_noir = 0
        self.nb_pieces_blanc = 0

        # Une liste qui store les 8 directions possibles du jeu.
        self.liste_directions = [(1, 0), (-1, 0), (0, 1), (0, -1), (1, 1), (-1, -1), (1, -1), (-1, 1)]

        # Appel de la méthode qui initialise une nouvelle planche.
        self.initialiser_planche_par_default()

        # Une pile avec tous les coups joués pendant la partie.
        self.historique_coups_jouees = []

    def get_piece(self, position):
        """
        Récupère une pièce dans la planche.

        Args:
            position: La position où récupérer la pièce, un tuple de coordonnées matricielles (ligne, colonne).

        Returns:
            La pièce à cette position s'il y en a une, None autrement.
        """
        if position in self.cases:
            return self.cases[position]
        else:
            return None

    def position_valide(self, position):
        """
        Vérifie si une position est valide (chaque coordonnée doit être dans les bornes).

        Args:
            position: Un couple (ligne, colonne), tuple de deux éléments.

        Returns:
            True si la position est valide, False autrement
        """
        return 0 <= position[0] < self.nb_cases and 0 <= position[1] < self.nb_cases

    def obtenir_positions_mangees(self, position, couleur):
        """
        Détermine quelles positions seront mangées si un coup de la couleur passée est joué à la position passée.

        Args:
            position: La position du coup à jouer.
            couleur: La couleur du coup à jouer.

        Returns:
            une liste contenant toutes les positions qui seraient mangées par le coup.
        """
        # Si la case est déjà occupée, retourne une liste vide.
        # Il est donc innécessaire d'appeler la méthode obtenir_positions_mangees_direction.
        if position in self.cases:
            return []

        # Une liste qui store toutes positions mangées.
        liste_positions_mangees = []

        # Nous appelons la méthode obtenir_positions_mangees_direction pour chacune des 8 directions.
        for direction in self.liste_directions:
            liste_positions_mangees += self.obtenir_positions_mangees_direction(couleur, direction, position)

        # Store le nombre de pièces mangées (utilisée par l'interface).
        if not liste_positions_mangees == []:
            self.nombres_pieces_mangees_par_case[position] = len(liste_positions_mangees)

        # Nous retournons la liste.
        return liste_positions_mangees

    def obtenir_positions_mangees_direction(self, couleur, direction, position):
        """
        Détermine les positions qui seront mangées si un coup de couleur "couleur" est joué à la position "position",
        si on parcourt la planche dans une direction "direction".

        Args:
            couleur: La couleur du coup évalué.
            direction: La direction de parcours évaluée.
            position: La position du coup évalué.

        Returns:
            La liste (peut-être vide) de toutes les positions mangées à partir du coup et de la direction donnés.
        """
        # Une liste qui va contenir toutes positions mangées dans une direction spécifique
        liste_positions_mangees = []

        while True:

            # On incrémente la position actuelle par une unité du vecteur directionel.
            position = position[0] + direction[0], position[1] + direction[1]

            # Si la position est invalide, retourne une liste vide.
            if not self.position_valide(position):
                return []

            # Si la case est vide, retourne une liste vide.
            elif position not in self.cases:
                return []

            # Si la pièce est de couleur différente, on l'ajoute à la liste.
            elif self.cases[position].couleur != couleur:
                liste_positions_mangees.append(position)

            # Si la pièce est de la même couleur, on retourne notre liste.
            elif self.cases[position].couleur == couleur:
                return liste_positions_mangees

    def coup_est_possible(self, position, couleur):
        """
        Détermine si un coup est possible. Un coup est possible si au moins une pièce est mangée par celui-ci.

        Args:
            position: La position du coup évalué.
            couleur: La couleur du coup évalué.

        Returns:
            True, si le coup est valide, False sinon.
        """
        return bool(self.obtenir_positions_mangees(position, couleur))

    def lister_coups_possibles_de_couleur(self, couleur):
        """
        Fonction retournant la liste des coups possibles d'une certaine couleur. Un coup est considéré possible
        si au moins une pièce est mangée quand la couleur "couleur" joue à une certaine position.

        Args:
            couleur: La couleur ("blanc", "noir") des pièces dont on considère le déplacement, un string.

        Returns:
            Une liste de positions de coups possibles pour la couleur "couleur".
        """
        liste_positions_coups_possibles = []
        self.nombres_pieces_mangees_par_case.clear()

        # Itération de chaque case du jeu.
        for ligne in range(0, self.nb_cases):
            for colonne in range(0, self.nb_cases):

                # Si le le coup à la case est possible, nous l'ajoutons à la liste.
                if (ligne, colonne) not in self.cases and self.coup_est_possible((ligne, colonne), couleur):
                    liste_positions_coups_possibles.append((ligne, colonne))

        return liste_positions_coups_possibles

    def jouer_coup(self, position, couleur):
        """
        Joue une pièce de la couleur "couleur" à la position "position".

        Args:
            position: La position du coup.
            couleur: La couleur du coup.
        """
        # Obtiens la liste de toutes positions mangées.
        liste_positions_mangees = self.obtenir_positions_mangees(position, couleur)

        self.positions_mangees_dernier_coup = liste_positions_mangees

        # Échange la couleur de chaque pièce mangée.
        for position_mangee in liste_positions_mangees:
            self.cases[position_mangee].echange_couleur()

        # Ajoute la nouvelle pièce sur la planche.
        self.cases[position] = Piece(couleur, position[0], position[1])

        # Store le coup dernièrement joué et les positions dernièrement mangées.
        self.historique_coups_jouees.append((position, list(self.positions_mangees_dernier_coup)))

    def convertir_en_chaine(self):
        """
        Retourne une chaîne de caractères où chaque case est écrite sur une ligne distincte.
        Chaque ligne contient l'information suivante:
        ligne,colonne,couleur

        Cette méthode pourrait par la suite être réutilisée pour sauvegarder une planche dans un fichier.

        Returns:
            La chaîne de caractères.
        """
        chaine = ""
        # Itération de chaque pièce sur la planche
        for piece in self.cases.values():
            chaine += "{},{},{}\n".format(piece.rangee, piece.colonne, piece.couleur)
        return chaine

    def charger_dune_chaine(self, chaine):
        """
        Remplit la planche à partir d'une chaîne de caractères comportant l'information d'une pièce sur chaque ligne.
        Chaque ligne contient l'information suivante:
        ligne,colonne,couleur

        Args:
            chaine: La chaîne de caractères, un string.
        """
        self.cases.clear()

        # Pour chaque ligne de la chaine.
        for ligne in chaine:

            # Split la ligne par les virgules.
            ligne_split = ligne.split(',')

            # Ajoute la pièce correspondante à la planche.
            rangee = int(ligne_split[0])
            colonne = int(ligne_split[1])
            couleur = ligne_split[2].rstrip()
            self.cases[rangee, colonne] = Piece(couleur, rangee, colonne)

        # Update le nombre de pièces.
        self.calculer_nombres_pieces()

    def initialiser_planche_par_default(self):
        """
        Initialise une planche de base avec la position initiale des pièces.
        """
        self.cases.clear()

        i = int(self.nb_cases / 2)
        j = i - 1

        self.cases[(j, j)] = Piece("blanc", j, j)
        self.cases[(j, i)] = Piece("noir", j, i)
        self.cases[(i, j)] = Piece("noir", i, j)
        self.cases[(i, i)] = Piece("blanc", i, i)

        # Update le nombre de pièces.
        self.calculer_nombres_pieces()

    def calculer_nombres_pieces(self):
        """
        Cette méthode update les variables qui représentent les nombres de pièces.
        """
        self.nb_pieces, self.nb_pieces_noir, self.nb_pieces_blanc = 0, 0, 0

        # Itération de chaque pièce sur la planche.
        for piece in self.cases.values():

            self.nb_pieces += 1

            # Si la la pièce est blanche:
            if piece.est_blanc():
                self.nb_pieces_blanc += 1

            # Si la pièce est noire:
            else:
                self.nb_pieces_noir += 1

        # Dessine le compteur des pièces noires et blanches.
        self.partie.fenetre.set_compteur_texte("Noir {} - {} Blanc".format(self.nb_pieces_noir, self.nb_pieces_blanc))

    def restaurer_cases(self):
        """
        Restaure l'état précédent des cases. Cela est fait en retrouvant l'élément sur le dessus de la pile.
        """
        coup_joue = self.historique_coups_jouees.pop()

        if coup_joue[0] is None:
            return

        # Échange la couleur des pièces dernièrement mangées.
        for position in coup_joue[1]:
            self.cases[position].echange_couleur()

        # Élimine la pièce dernièrement ajoutée.
        del self.cases[coup_joue[0]]

        self.partie.coups_possibles = self.lister_coups_possibles_de_couleur(self.partie.joueur_courant.couleur)
