class Simulateur:
    """
    Un object simulateur gère l'exécution de l'algorithme minimax pour chosisir un coup avantageux.
    """
    def __init__(self, couleur_joueur, planche):
        self.couleur_joueur = couleur_joueur
        self.planche = planche
        self.nb_cases = self.planche.nb_cases

        if couleur_joueur == "noir":
            self.couleur_adversaire = "blanc"
        else:
            self.couleur_adversaire = "noir"

        # Construit une table de pointage par rapport au nombre de cases de la planche.
        self.table_de_pointage = self.construire_table_de_pointage()

    def construire_table_de_pointage(self):
        """
        Crée une table de pointage selon la dimenion de la planche.
        La valeur à chaque position représente l'importance de la case.
        Par exemple un 100 représente une case très importante en ordre de priorité.

        EXAMPLE DE TABLE DE POINTAGE POUR dimension_planche == 8:
        [[100,  -1,  5,  2,  2,  5,  -1, 100],
        [  -1, -20,  1,  1,  1,  1, -20,  -1],
        [   5,   1,  1,  1,  1,  1,   1,   5],
        [   2,   1,  1,  0,  0,  1,   1,   2],
        [   2,   1,  1,  0,  0,  1,   1,   2],
        [   5,   1,  1,  1,  1,  1,   1,   5],
        [  -1, -20,  1,  1,  1,  1, -20,  -1],
        [ 100,  -1,  5,  2,  2,  5,  -1, 100]]

        Returns:
            une table de pointage, soit une liste comportante n listes de n integers
        """

        dimension_planche = self.nb_cases

        indice_max = dimension_planche - 1
        table_de_pointage = []

        # Initialise la table de pointage.
        for i in range(0, dimension_planche):
            table_de_pointage.append([0] * dimension_planche)

        # Disposition des points selon leur position attitré dans la table de pointage.
        for i in range(0, dimension_planche):
            for j in range(0, dimension_planche):

                # Les 4 coins recoivent 100 points.
                if (i, j) == (0, 0) \
                        or (i, j) == (0, indice_max) \
                        or (i, j) == (indice_max, 0) \
                        or (i, j) == (indice_max, indice_max):

                    table_de_pointage[i][j] = 100
                    continue

                # Les 8 cases adjacantes aux 4 coins recoivent -1 points.
                if (i, j) == (0, 1) \
                        or (i, j) == (0, indice_max - 1) \
                        or (i, j) == (1, 0) \
                        or (i, j) == (1, indice_max) \
                        or (i, j) == (indice_max - 1, 0) \
                        or (i, j) == (indice_max, 1) \
                        or (i, j) == (indice_max - 1, indice_max) \
                        or (i, j) == (indice_max, indice_max - 1):

                    table_de_pointage[i][j] = -1
                    continue

                # Les 8 cases adjacantes aux 8 précédentes recoivent 5 points.
                if (i, j) == (0, 2) \
                        or (i, j) == (0, indice_max - 2) \
                        or (i, j) == (2, 0) \
                        or (i, j) == (2, indice_max) \
                        or (i, j) == (indice_max - 2, 0) \
                        or (i, j) == (indice_max, 2) \
                        or (i, j) == (indice_max - 2, indice_max) \
                        or (i, j) == (indice_max, indice_max - 2):

                    table_de_pointage[i][j] = 5
                    continue

                # Les 4 cases en diagonale avec les 4 coins recoivent -20 points.
                if (i, j) == (1, 1) \
                        or (i, j) == (1, indice_max - 1) \
                        or (i, j) == (indice_max - 1, 1) \
                        or (i, j) == (indice_max - 1, indice_max - 1):

                    table_de_pointage[i][j] = -20
                    continue

                # Le restant des cases sur le périmètre recoivent 2 points.
                if i == 0\
                        or i == indice_max \
                        or j == 0 \
                        or j == indice_max:

                    table_de_pointage[i][j] = 2
                    continue

                # Les 4 cases centrales recoivent 0 points.
                if (i, j) == (dimension_planche / 2 - 1, dimension_planche / 2) \
                        or (i, j) == (dimension_planche / 2, dimension_planche / 2) \
                        or (i, j) == (dimension_planche / 2 - 1, dimension_planche / 2 - 1) \
                        or (i, j) == (dimension_planche / 2, dimension_planche / 2 - 1):

                    table_de_pointage[i][j] = 0
                    continue

                # Le restant des cases recoivent 1 point.
                table_de_pointage[i][j] = 1

        return table_de_pointage

    def calculer_pointage_selon_tableau(self):
        """
        Calcule le pointage pour l'état actuel de la planche.

        Returns:
            Un nombre entier qui représente le pointage de la planche actuelle.
        """
        pointage = 0
        mes_pieces = 0
        pieces_adversaire = 0

        # Parcourt le tableau de pointage selon le positionnement des pieces.
        # Si la case contient une pièce du joueur: incrémente le pointage.
        # Si la case contient une pièce de l'adversaire: décrémente le pointage.
        for piece in self.planche.cases.values():
            if piece.couleur == self.couleur_joueur:
                mes_pieces += 1
                pointage += self.table_de_pointage[piece.rangee][piece.colonne]
            else:
                pieces_adversaire += 1
                pointage -= self.table_de_pointage[piece.rangee][piece.colonne]

        # Si le joueur à gagné, retourne 1000.
        # Si le joueur à perdu, retourne -1000.
        # Sinon retourne le pointage.
        if mes_pieces == 0:
            return -1000
        if pieces_adversaire == 0:
            return 1100
        if mes_pieces + pieces_adversaire == self.planche.nb_cases**2:
            if mes_pieces > pieces_adversaire:
                return 1000 + mes_pieces
            else:
                return -1000 + mes_pieces
        return pointage

    def minimax(self, coup, maximizer, profondeur, alpha, beta):
        """
        Exécute des simulations pour chaque possibilité de jeu pour un nombre limité de coups.
        Assigne une valeur à chaque simulation puis retourne la valeur maximale si c'est le tour au joueur
        ou la valeur minimale si c'est le tour à l'adversaire.

        Args:
            coup: une position (x, y) à jouer pour la simulation.
            maximizer: True si le maximizer est en train de jouer, False si c'est la minimizer.
            profondeur: le nombre de profondeurs qui restent à être simulées.
            alpha: le pointage maximum découvert.
            beta: le pointage minimum découvert.

        Returns:
            Un nombre entier qui représente le pointage de la planche actuelle.
        """
        if profondeur < 1:
            print("ERREUR: compteur minimax < 1. TERMINATION")
            exit(1)

        # Joue un coup s'il existe.
        if coup:
            if maximizer:
                self.planche.jouer_coup(coup, self.couleur_adversaire)
            else:
                self.planche.jouer_coup(coup, self.couleur_joueur)
        else:
            self.planche.historique_coups_jouees.append((None, None))

        # Si nous avons atteint la profondeur désirée, retourne le pointage et restaure les cases.
        if profondeur == 1:
            pointage = self.calculer_pointage_selon_tableau()
            self.planche.restaurer_cases()
            return pointage

        # MAXIMIZER
        if maximizer:

            coups_possibles = self.planche.lister_coups_possibles_de_couleur(self.couleur_joueur)

            # Si aucun coup possible continue la simulation.
            if not coups_possibles:

                pointage = self.minimax(None, False, profondeur - 1, alpha, beta)
                self.planche.restaurer_cases()
                return pointage

            pointage_maximum = -3000

            for coup in coups_possibles:

                pointage_maximum = max(pointage_maximum, self.minimax(coup, False, profondeur - 1, alpha, beta))

                alpha = max(alpha, pointage_maximum)

                if alpha >= beta:
                    break

            self.planche.restaurer_cases()
            return pointage_maximum

        # MINIMIZER
        else:

            coups_possibles = self.planche.lister_coups_possibles_de_couleur(self.couleur_adversaire)

            # Si aucun coup possible continue la simulation.
            if not coups_possibles:
                pointage = self.minimax(None, True, profondeur - 1, alpha, beta)

                self.planche.restaurer_cases()
                return pointage

            pointage_minimum = 3000

            for coup in coups_possibles:

                pointage_minimum = min(pointage_minimum, self.minimax(coup, True, profondeur - 1, alpha, beta))

                beta = min(beta, pointage_minimum)

                if alpha >= beta:
                    break

            self.planche.restaurer_cases()
            return pointage_minimum
